@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -678,13 +678,6 @@ public final class XChoicePeer extends XComponentPeer implements ChoicePeer, Top
         repaint();
     }
 
-    /**
-     * DEPRECATED: Replaced by add(String, int).
-     */
-    public void addItem(String item, int index) {
-        add(item, index);
-    }
-
     public void setFont(Font font) {
         super.setFont(font);
         helper.setFont(this.font);
