@@ -33,6 +33,11 @@
 #include "jni_util.h"
 #include "jdk_net_LinuxSocketOptions.h"
 
+/*
+ * Declare library specific JNI_Onload entry if static build
+ */
+DEF_STATIC_JNI_OnLoad
+
 /*
  * Class:     jdk_net_LinuxSocketOptions
  * Method:    setQuickAck
