@@ -21,63 +21,64 @@
  * questions.
  */
 
+package gc.epsilon;
+
 /**
  * @test TestRefArrays
- * @key gc
+ * @key gc randomness
  * @requires vm.gc.Epsilon & !vm.graal.enabled
  * @summary Epsilon is able to allocate arrays, and does not corrupt their state
  * @library /test/lib
  *
  * @run main/othervm -XX:+UseTLAB -Xmx256m
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:+UseTLAB -Xmx256m
  *                   -Xint
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:+UseTLAB -Xmx256m
  *                   -Xbatch -Xcomp -XX:TieredStopAtLevel=1
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:+UseTLAB -Xmx256m
  *                   -Xbatch -Xcomp -XX:-TieredCompilation
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:-UseTLAB -Xmx256m
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:-UseTLAB -Xmx256m
  *                   -Xint
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:-UseTLAB -Xmx256m
  *                   -Xbatch -Xcomp -XX:TieredStopAtLevel=1
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  *
  * @run main/othervm -XX:-UseTLAB -Xmx256m
  *                   -Xbatch -Xcomp -XX:-TieredCompilation
  *                   -XX:+UnlockExperimentalVMOptions -XX:+UseEpsilonGC
- *                   TestRefArrays
+ *                   gc.epsilon.TestRefArrays
  */
 
 import java.util.Random;
+import jdk.test.lib.Utils;
 
 public class TestRefArrays {
-
-  static long SEED = Long.getLong("seed", System.nanoTime());
   static int COUNT = Integer.getInteger("count", 200); // ~100 MB allocation
 
   static MyObject[][] arr;
 
   public static void main(String[] args) throws Exception {
-    Random r = new Random(SEED);
+    Random r = Utils.getRandomInstance();
 
     arr = new MyObject[COUNT * 100][];
     for (int c = 0; c < COUNT; c++) {
@@ -87,7 +88,7 @@ public class TestRefArrays {
       }
     }
 
-    r = new Random(SEED);
+    r = new Random(Utils.SEED);
     for (int c = 0; c < COUNT; c++) {
       MyObject[] b = arr[c];
       if (b.length != (c * 100)) {
