@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,7 +24,8 @@
 /*
  * @test
  * @bug 4028006 4044013 4096694 4107276 4107570 4112869 4130885 7039469 7126465 7158483
- *      8008577 8077685 8098547 8133321 8138716 8148446 8151876 8159684 8166875
+ *      8008577 8077685 8098547 8133321 8138716 8148446 8151876 8159684 8166875 8181157
+ *      8228469 8274407 8305113
  * @modules java.base/sun.util.resources
  * @library /java/text/testlib
  * @summary test TimeZone
@@ -101,7 +102,7 @@ public class TimeZoneTest extends IntlTest
     public void TestShortZoneIDs() throws Exception {
 
         ZoneDescriptor[] JDK_116_REFERENCE_LIST = {
-            new ZoneDescriptor("MIT", 780, true),
+            new ZoneDescriptor("MIT", 780, false), // Samoa no longer observes DST starting 2021b
             new ZoneDescriptor("HST", -600, false),
             new ZoneDescriptor("AST", -540, true),
             new ZoneDescriptor("PST", -480, true),
@@ -113,14 +114,14 @@ public class TimeZoneTest extends IntlTest
             new ZoneDescriptor("PRT", -240, false),
             new ZoneDescriptor("CNT", -210, true),
             new ZoneDescriptor("AGT", -180, false),
-            new ZoneDescriptor("BET", -180, true),
+            new ZoneDescriptor("BET", -180, false),
             // new ZoneDescriptor("CAT", -60, false), // Wrong:
             // As of bug 4130885, fix CAT (Central Africa)
             new ZoneDescriptor("CAT", 120, false), // Africa/Harare
             new ZoneDescriptor("GMT", 0, false),
             new ZoneDescriptor("UTC", 0, false),
             new ZoneDescriptor("ECT", 60, true),
-            new ZoneDescriptor("ART", 120, false),
+            new ZoneDescriptor("ART", 120, true),
             new ZoneDescriptor("EET", 120, true),
             new ZoneDescriptor("EAT", 180, false),
             new ZoneDescriptor("MET", 60, true),
@@ -364,6 +365,7 @@ public class TimeZoneTest extends IntlTest
         }
         else if (!name.equals("Pacific Standard Time") &&
                  !name.equals("\u592a\u5e73\u6d0b\u6807\u51c6\u65f6\u95f4") &&
+                 !name.equals("\u5317\u7f8e\u592a\u5e73\u6d0b\u6807\u51c6\u65f6\u95f4") &&
                  !name.equals("GMT-08:00") &&
                  !name.equals("GMT-8:00") &&
                  !name.equals("GMT-0800") &&
@@ -504,234 +506,4 @@ public class TimeZoneTest extends IntlTest
     {
         // ErrorCode status = ZERO_ERROR;
         boolean startsInDST = tz.inDaylightTime(new Date(min));
-        // if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
-        if (tz.inDaylightTime(new Date(max)) == startsInDST) {
-            logln("Error: inDaylightTime(" + new Date(max) + ") != " + (!startsInDST));
-            return;
-        }
-        // if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
-        while ((max - min) > INTERVAL) {
-            long mid = (min + max) / 2;
-            if (tz.inDaylightTime(new Date(mid)) == startsInDST) {
-                min = mid;
-            }
-            else {
-                max = mid;
-            }
-            // if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
-        }
-        logln("Binary Search Before: " + min + " = " + new Date(min));
-        logln("Binary Search After:  " + max + " = " + new Date(max));
-        long mindelta = expectedBoundary - min;
-        long maxdelta = max - expectedBoundary;
-        if (mindelta >= 0 &&
-            mindelta <= INTERVAL &&
-            mindelta >= 0 &&
-            mindelta <= INTERVAL)
-            logln("PASS: Expected bdry:  " + expectedBoundary + " = " + new Date(expectedBoundary));
-        else
-            errln("FAIL: Expected bdry:  " + expectedBoundary + " = " + new Date(expectedBoundary));
-    }
-
-    static final int INTERVAL = 100;
-
-    // Bug 006; verify the offset for a specific zone.
-    public void TestPRTOffset()
-    {
-        TimeZone tz = TimeZone.getTimeZone( "PRT" );
-        if( tz == null ) {
-            errln( "FAIL: TimeZone(PRT) is null" );
-        }
-        else{
-            if (tz.getRawOffset() != (-4*millisPerHour))
-                errln("FAIL: Offset for PRT should be -4");
-        }
-
-    }
-
-    // Test various calls
-    @SuppressWarnings("deprecation")
-    public void TestVariousAPI518()
-    {
-        TimeZone time_zone = TimeZone.getTimeZone("PST");
-        Date d = new Date(97, Calendar.APRIL, 30);
-
-        logln("The timezone is " + time_zone.getID());
-
-        if (time_zone.inDaylightTime(d) != true)
-            errln("FAIL: inDaylightTime returned false");
-
-        if (time_zone.useDaylightTime() != true)
-            errln("FAIL: useDaylightTime returned false");
-
-        if (time_zone.getRawOffset() != -8*millisPerHour)
-            errln( "FAIL: getRawOffset returned wrong value");
-
-        GregorianCalendar gc = new GregorianCalendar();
-        gc.setTime(d);
-        if (time_zone.getOffset(gc.AD, gc.get(gc.YEAR), gc.get(gc.MONTH),
-                                gc.get(gc.DAY_OF_MONTH),
-                                gc.get(gc.DAY_OF_WEEK), 0)
-            != -7*millisPerHour)
-            errln("FAIL: getOffset returned wrong value");
-    }
-
-    // Test getAvailableID API
-    public void TestGetAvailableIDs913()
-    {
-        StringBuffer buf = new StringBuffer("TimeZone.getAvailableIDs() = { ");
-        String[] s = TimeZone.getAvailableIDs();
-        for (int i=0; i<s.length; ++i)
-        {
-            if (i > 0) buf.append(", ");
-            buf.append(s[i]);
-        }
-        buf.append(" };");
-        logln(buf.toString());
-
-        buf.setLength(0);
-        buf.append("TimeZone.getAvailableIDs(GMT+02:00) = { ");
-        s = TimeZone.getAvailableIDs(+2 * 60 * 60 * 1000);
-        for (int i=0; i<s.length; ++i)
-        {
-            if (i > 0) buf.append(", ");
-            buf.append(s[i]);
-        }
-        buf.append(" };");
-        logln(buf.toString());
-
-        TimeZone tz = TimeZone.getTimeZone("PST");
-        if (tz != null)
-            logln("getTimeZone(PST) = " + tz.getID());
-        else
-            errln("FAIL: getTimeZone(PST) = null");
-
-        tz = TimeZone.getTimeZone("America/Los_Angeles");
-        if (tz != null)
-            logln("getTimeZone(America/Los_Angeles) = " + tz.getID());
-        else
-            errln("FAIL: getTimeZone(PST) = null");
-
-        // Bug 4096694
-        tz = TimeZone.getTimeZone("NON_EXISTENT");
-        if (tz == null)
-            errln("FAIL: getTimeZone(NON_EXISTENT) = null");
-        else if (!tz.getID().equals("GMT"))
-            errln("FAIL: getTimeZone(NON_EXISTENT) = " + tz.getID());
-    }
-
-    /**
-     * Bug 4107276
-     */
-    public void TestDSTSavings() {
-        // It might be better to find a way to integrate this test into the main TimeZone
-        // tests above, but I don't have time to figure out how to do this (or if it's
-        // even really a good idea).  Let's consider that a future.  --rtg 1/27/98
-        SimpleTimeZone tz = new SimpleTimeZone(-5 * millisPerHour, "dstSavingsTest",
-                                               Calendar.MARCH, 1, 0, 0, Calendar.SEPTEMBER, 1, 0, 0,
-                                               (int)(0.5 * millisPerHour));
-
-        if (tz.getRawOffset() != -5 * millisPerHour)
-            errln("Got back a raw offset of " + (tz.getRawOffset() / millisPerHour) +
-                  " hours instead of -5 hours.");
-        if (!tz.useDaylightTime())
-            errln("Test time zone should use DST but claims it doesn't.");
-        if (tz.getDSTSavings() != 0.5 * millisPerHour)
-            errln("Set DST offset to 0.5 hour, but got back " + (tz.getDSTSavings() /
-                                                                 millisPerHour) + " hours instead.");
-
-        int offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JANUARY, 1,
-                                  Calendar.THURSDAY, 10 * millisPerHour);
-        if (offset != -5 * millisPerHour)
-            errln("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JUNE, 1, Calendar.MONDAY,
-                              10 * millisPerHour);
-        if (offset != -4.5 * millisPerHour)
-            errln("The offset for 10 AM, 6/1/98 should have been -4.5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        tz.setDSTSavings(millisPerHour);
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JANUARY, 1,
-                              Calendar.THURSDAY, 10 * millisPerHour);
-        if (offset != -5 * millisPerHour)
-            errln("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JUNE, 1, Calendar.MONDAY,
-                              10 * millisPerHour);
-        if (offset != -4 * millisPerHour)
-            errln("The offset for 10 AM, 6/1/98 (with a 1-hour DST offset) should have been -4 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-    }
-
-    /**
-     * Bug 4107570
-     */
-    public void TestAlternateRules() {
-        // Like TestDSTSavings, this test should probably be integrated somehow with the main
-        // test at the top of this class, but I didn't have time to figure out how to do that.
-        //                      --rtg 1/28/98
-
-        SimpleTimeZone tz = new SimpleTimeZone(-5 * millisPerHour, "alternateRuleTest");
-
-        // test the day-of-month API
-        tz.setStartRule(Calendar.MARCH, 10, 12 * millisPerHour);
-        tz.setEndRule(Calendar.OCTOBER, 20, 12 * millisPerHour);
-
-        int offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 5,
-                                  Calendar.THURSDAY, 10 * millisPerHour);
-        if (offset != -5 * millisPerHour)
-            errln("The offset for 10AM, 3/5/98 should have been -5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 15,
-                              Calendar.SUNDAY, 10 * millisPerHour);
-        if (offset != -4 * millisPerHour)
-            errln("The offset for 10AM, 3/15/98 should have been -4 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 15,
-                              Calendar.THURSDAY, 10 * millisPerHour);
-        if (offset != -4 * millisPerHour)
-            errln("The offset for 10AM, 10/15/98 should have been -4 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 25,
-                              Calendar.SUNDAY, 10 * millisPerHour);
-        if (offset != -5 * millisPerHour)
-            errln("The offset for 10AM, 10/25/98 should have been -5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        // test the day-of-week-after-day-in-month API
-        tz.setStartRule(Calendar.MARCH, 10, Calendar.FRIDAY, 12 * millisPerHour, true);
-        tz.setEndRule(Calendar.OCTOBER, 20, Calendar.FRIDAY, 12 * millisPerHour, false);
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 11,
-                              Calendar.WEDNESDAY, 10 * millisPerHour);
-        if (offset != -5 * millisPerHour)
-            errln("The offset for 10AM, 3/11/98 should have been -5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 14,
-                              Calendar.SATURDAY, 10 * millisPerHour);
-        if (offset != -4 * millisPerHour)
-            errln("The offset for 10AM, 3/14/98 should have been -4 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 15,
-                              Calendar.THURSDAY, 10 * millisPerHour);
-        if (offset != -4 * millisPerHour)
-            errln("The offset for 10AM, 10/15/98 should have been -4 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-
-        offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 17,
-                              Calendar.SATURDAY, 10 * millisPerHour);
-        if (offset != -5 * millisPerHour)
-            errln("The offset for 10AM, 10/17/98 should have been -5 hours, but we got "
-                  + (offset / millisPerHour) + " hours.");
-    }
-}
-
-//eof
+        // if (failure(status, 
\ No newline at end of file
