@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -84,6 +84,12 @@ abstract public class SSLEngineTestCase {
         ENABLED_NON_KRB_NOT_ANON_CIPHERS(
                 SSLEngineTestCase.ENABLED_NON_KRB_NOT_ANON_CIPHERS,
                 "Enabled by default non kerberos not anonymous"),
+        /**
+         * Ciphers supported by TLS 1.3 only.
+         */
+        TLS13_CIPHERS(
+                SSLEngineTestCase.TLS13_CIPHERS,
+                "Supported by TLS 1.3 only"),
         /**
          * Ciphers unsupported by the tested SSLEngine.
          */
@@ -128,7 +134,7 @@ abstract public class SSLEngineTestCase {
      * e.g. "TLSv1", "TLSv1.1", "TLSv1.2", "DTLSv1.0", "DTLSv1.2".
      */
     public static final String TESTED_SECURITY_PROTOCOL
-            = System.getProperty("test.security.protocol", "TLS");
+            = System.getProperty("test.security.protocol");
     /**
      * Test mode: "norm", "norm_sni" or "krb".
      * Modes "norm" and "norm_sni" are used to run
@@ -174,6 +180,12 @@ abstract public class SSLEngineTestCase {
     private static final String SERVER_NAME = "service.localhost";
     private static final String SNI_PATTERN = ".*";
 
+    private static final String[] TLS13_CIPHERS = {
+            "TLS_AES_256_GCM_SHA384",
+            "TLS_AES_128_GCM_SHA256",
+            "TLS_CHACHA20_POLY1305_SHA256"
+    };
+
     private static final String[] SUPPORTED_NON_KRB_CIPHERS;
 
     static {
@@ -183,8 +195,8 @@ abstract public class SSLEngineTestCase {
             List<String> supportedCiphersList = new LinkedList<>();
             for (String cipher : allSupportedCiphers) {
                 if (!cipher.contains("KRB5")
-                    && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")) {
-
+                        && !isTLS13Cipher(cipher)
+                        && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")) {
                     supportedCiphersList.add(cipher);
                 }
             }
@@ -204,6 +216,7 @@ abstract public class SSLEngineTestCase {
             List<String> supportedCiphersList = new LinkedList<>();
             for (String cipher : allSupportedCiphers) {
                 if (!cipher.contains("KRB5")
+                        && !isTLS13Cipher(cipher)
                         && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")
                         && !cipher.endsWith("_SHA256")
                         && !cipher.endsWith("_SHA384")) {
@@ -226,7 +239,8 @@ abstract public class SSLEngineTestCase {
             List<String> supportedCiphersList = new LinkedList<>();
             for (String cipher : allSupportedCiphers) {
                 if (cipher.contains("KRB5")
-                    && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")) {
+                        && !isTLS13Cipher(cipher)
+                        && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")) {
                     supportedCiphersList.add(cipher);
                 }
             }
@@ -246,7 +260,8 @@ abstract public class SSLEngineTestCase {
             List<String> enabledCiphersList = new LinkedList<>();
             for (String cipher : enabledCiphers) {
                 if (!cipher.contains("anon") && !cipher.contains("KRB5")
-                    && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")) {
+                        && !isTLS13Cipher(cipher)
+                        && !cipher.contains("TLS_EMPTY_RENEGOTIATION_INFO_SCSV")) {
                     enabledCiphersList.add(cipher);
                 }
             }
@@ -303,6 +318,16 @@ abstract public class SSLEngineTestCase {
         this.maxPacketSize = 0;
     }
 
+    private static boolean isTLS13Cipher(String cipher) {
+        for (String cipherSuite : TLS13_CIPHERS) {
+            if (cipherSuite.equals(cipher)) {
+                return true;
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Wraps data with the specified engine.
      *
@@ -392,7 +417,7 @@ abstract public class SSLEngineTestCase {
         int length = net.remaining();
         System.out.println(wrapper + " wrapped " + length + " bytes.");
         System.out.println(wrapper + " handshake status is "
-                + engine.getHandshakeStatus());
+                + engine.getHandshakeStatus() + " Result is " + r.getStatus());
         if (maxPacketSize < length && maxPacketSize != 0) {
             throw new AssertionError("Handshake wrapped net buffer length "
                     + length + " exceeds maximum packet size "
@@ -480,7 +505,7 @@ abstract public class SSLEngineTestCase {
         SSLEngineResult r = engine.unwrap(net, app);
         app.flip();
         System.out.println(unwrapper + " handshake status is "
-                + engine.getHandshakeStatus());
+                + engine.getHandshakeStatus() + " Result is " + r.getStatus());
         checkResult(r, wantedStatus);
         if (result != null && result.length > 0) {
             result[0] = r;
@@ -713,8 +738,18 @@ abstract public class SSLEngineTestCase {
                     case "TLSv1.1":
                         runTests(Ciphers.SUPPORTED_NON_KRB_NON_SHA_CIPHERS);
                         break;
-                    default:
+                    case "DTLS":
+                    case "DTLSv1.2":
+                    case "TLS":
+                    case "TLSv1.2":
                         runTests(Ciphers.SUPPORTED_NON_KRB_CIPHERS);
+                        break;
+                    case "TLSv1.3":
+                        runTests(Ciphers.TLS13_CIPHERS);
+                        break;
+                    default:
+                        throw new Error("Test error: Unsupported test " +
+                                "security protocol: " + TESTED_SECURITY_PROTOCOL);
                 }
                 break;
             case "krb":
@@ -907,6 +942,7 @@ abstract public class SSLEngineTestCase {
                 case SUPPORTED_NON_KRB_NON_SHA_CIPHERS:
                 case SUPPORTED_KRB_CIPHERS:
                 case ENABLED_NON_KRB_NOT_ANON_CIPHERS:
+                case TLS13_CIPHERS:
                     if (error != null) {
                         System.out.println("Test Failed: " + cs);
                         System.err.println("Test Exception for " + cs);
