@@ -27,5 +27,5 @@ import java.io.FileNotFoundException;
 import java.nio.file.Path;
 
 public interface ArtifactManager {
-    public Path resolve(Artifact artifact) throws FileNotFoundException;
+    public Path resolve(Artifact artifact) throws ArtifactResolverException;
 }
