@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,52 +22,13 @@
  */
 
 /**
- * JDK-8023368: Instance __proto__ property should exist and be writable.
- *
+ * JDK-8048079: Persistent code store is broken after optimistic types merge
+ * Same script as JDK-8048079_1b.js to exercise code cache again.
  * @test
- * @run
+ * @runif external.yui
+ * @option -pcc
+ * @option -Dnashorn.persistent.code.cache=build/nashorn_code_cache
+ * @fork
  */
 
-// check Object.setPrototypeOf extension rather than using __proto__
-
-// function to force same callsites
-function check(obj) {
-    print(obj.func());
-    print(obj.x);
-    print(obj.toString());
-}
-
-function Func() {
-}
-
-Func.prototype.func = function() {
-    return "Func.prototype.func";
-}
-
-Func.prototype.x = "hello";
-
-var obj = new Func();
-var obj2 = Object.create(obj);
-
-// check direct and indirect __proto__ change
-check(obj);
-check(obj2);
-Object.setPrototypeOf(obj, {
-   func: function() {
-        return "obj.__proto__.func @ " + __LINE__;
-   },
-   x: 344
-});
-
-check(obj);
-check(obj2);
-
-// check indirect (1 and 2 levels) __proto__ function change
-Object.setPrototypeOf(Object.getPrototypeOf(obj), {
-    toString: function() {
-        return "new object.toString";
-    }
-});
-
-check(obj);
-check(obj2);
+load(__DIR__ + 'yui.js');
